@@ -6,10 +6,11 @@ from copy import deepcopy
 from functools import partial
 from types import FunctionType
 from typing import Type, Tuple, Sequence, Dict, Callable, Any, Optional, List
+
+import ujson
 from django.apps import apps
 from django.conf.urls import url
 from django.db.models import QuerySet, Model
-from drf_json_api_utils.common import DEFAULT_PAGE_SIZE, exception_handler
 from rest_framework.authentication import BaseAuthentication
 from rest_framework.parsers import FormParser, MultiPartParser
 from rest_framework.permissions import BasePermission
@@ -21,6 +22,8 @@ from rest_framework_json_api.metadata import JSONAPIMetadata
 from rest_framework_json_api.parsers import JSONParser
 from rest_framework_json_api.renderers import JSONRenderer
 from rest_framework_json_api.views import RelationshipView, ModelViewSet
+
+from drf_json_api_utils.common import DEFAULT_PAGE_SIZE, exception_handler
 from . import json_api_spec_http_methods
 from . import lookups as filter_lookups
 from . import plugins
@@ -71,6 +74,9 @@ def get_dict_by_methods(view_type, allowed_http_methods):
     return out
 
 
+DEFAULT_VOID_DECORATOR = lambda func: func
+
+
 class JsonApiModelViewBuilder:
     DEFAULT_RELATED_LIMIT = 100
 
@@ -88,13 +94,18 @@ class JsonApiModelViewBuilder:
         '_authentication_classes': {'kwarg': 'authentication_classes', 'default': []},
         '_before_create_callback': {'kwarg': None, 'default': None},
         '_after_create_callback': {'kwarg': None, 'default': None},
+        '_create_decorator': {'kwarg': None, 'default': DEFAULT_VOID_DECORATOR},
         '_after_get_callback': {'kwarg': None, 'default': None},
+        '_get_decorator': {'kwarg': None, 'default': DEFAULT_VOID_DECORATOR},
         '_before_update_callback': {'kwarg': None, 'default': None},
         '_after_update_callback': {'kwarg': None, 'default': None},
+        '_update_decorator': {'kwarg': None, 'default': DEFAULT_VOID_DECORATOR},
         '_before_delete_callback': {'kwarg': None, 'default': None},
         '_after_delete_callback': {'kwarg': None, 'default': None},
+        '_delete_decorator': {'kwarg': None, 'default': DEFAULT_VOID_DECORATOR},
         '_before_list_callback': {'kwarg': None, 'default': None},
         '_after_list_callback': {'kwarg': None, 'default': None},
+        '_list_decorator': {'kwarg': None, 'default': DEFAULT_VOID_DECORATOR},
         '_before_raw_response': {'kwarg': None, 'default': None},
         '_expose_related_views': {'kwarg': 'expose_related_views', 'default': None},
         '_is_admin': {'kwarg': 'is_admin', 'default': None},
@@ -106,6 +117,7 @@ class JsonApiModelViewBuilder:
         '_primary_key_name': {'kwarg': 'primary_key_name', 'default': 'id'},
         '_resource_name': {'kwarg': 'resource_name', 'default': None},
         '_page_size': {'kwarg': 'page_size', 'default': None},
+        '_dummy_includes': {'kwarg': 'dummy_includes', 'default': []},
     }
 
     def __init__(self, model: Type[Model],
@@ -121,7 +133,8 @@ class JsonApiModelViewBuilder:
                  plugin_options: Optional[Dict[str, Any]] = None,
                  expose_related_views: Optional[bool] = False,
                  is_admin: Optional[bool] = False,
-                 always_include: Optional[bool] = False):
+                 always_include: Optional[bool] = False,
+                 page_size: Optional[int] = DEFAULT_PAGE_SIZE):
         self.override(
             model=model,
             primary_key_name=primary_key_name,
@@ -137,6 +150,7 @@ class JsonApiModelViewBuilder:
             expose_related_views=expose_related_views,
             is_admin=is_admin,
             always_include=always_include,
+            page_size=page_size
         )
         self.settings = JsonApiGlobalSettings()
 
@@ -247,6 +261,10 @@ class JsonApiModelViewBuilder:
                                                       filter_type=filter_type)
         return self
 
+    def add_dummy_include(self, field_name):
+        self._dummy_includes.append(field_name)
+        return self
+
     def add_relation(self, field: str, many: bool = False, resource_name: str = None,
                      primary_key_name: str = None,
                      limit_to_on_retrieve: bool = False,
@@ -312,11 +330,22 @@ class JsonApiModelViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_POST)
         return self
 
+    def set_create_decorator(self,
+                             create_decorator: Callable[[Callable], Callable] = None) -> 'JsonApiModelViewBuilder':
+        self._create_decorator = create_decorator
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def after_get(self, after_get_callback: Callable[[Any], Any] = None) -> 'JsonApiModelViewBuilder':
         self._after_get_callback = after_get_callback
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
         return self
 
+    def set_get_decorator(self, get_decorator: Callable[[Callable], Callable] = None) -> 'JsonApiModelViewBuilder':
+        self._get_decorator = get_decorator
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def before_update(self, before_update_callback: Callable[[Any], Any] = None) -> 'JsonApiModelViewBuilder':
         self._before_update_callback = before_update_callback
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_PATCH)
@@ -327,6 +356,12 @@ class JsonApiModelViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_PATCH)
         return self
 
+    def set_update_decorator(self,
+                             update_decorator: Callable[[Callable], Callable] = None) -> 'JsonApiModelViewBuilder':
+        self._update_decorator = update_decorator
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def before_delete(self, before_delete_callback: Callable[[Any], Any] = None) -> 'JsonApiModelViewBuilder':
         self._before_delete_callback = before_delete_callback
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_DELETE)
@@ -337,6 +372,12 @@ class JsonApiModelViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_DELETE)
         return self
 
+    def set_delete_decorator(self,
+                             delete_decorator: Callable[[Callable], Callable] = None) -> 'JsonApiModelViewBuilder':
+        self._delete_decorator = delete_decorator
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def before_list(self,
                     before_list_callback: Callable[[Request, QuerySet], QuerySet] = None) -> 'JsonApiModelViewBuilder':
         self._before_list_callback = before_list_callback
@@ -348,6 +389,11 @@ class JsonApiModelViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
         return self
 
+    def set_list_decorator(self, list_decorator: Callable[[Callable], Callable] = None) -> 'JsonApiModelViewBuilder':
+        self._list_decorator = list_decorator
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def before_response(self, before_raw_response: Callable[[str], str] = None) -> 'JsonApiModelViewBuilder':
         self._before_raw_response = before_raw_response
         return self
@@ -412,7 +458,8 @@ class JsonApiModelViewBuilder:
                                           self._primary_key_name,
                                           self._before_update_callback if limit_to_on_retrieve else self._before_create_callback,
                                           self._after_list_callback,
-                                          self._is_admin)
+                                          self._is_admin,
+                                          self._dummy_includes)
                 _append_to_namespace(method_to_serializer[limit_to_on_retrieve])
         else:
 
@@ -427,12 +474,14 @@ class JsonApiModelViewBuilder:
                                                                self._computed_filters)
 
         @exception_handler
+        @self._create_decorator
         def perform_create(view, serializer):
             instance = serializer.save()
             if self._after_create_callback is not None:
                 self._after_create_callback(view.request, instance, serializer)
 
         @exception_handler
+        @self._delete_decorator
         def perform_destroy(view, instance):
             if self._before_delete_callback is not None:
                 self._before_delete_callback(instance, view.get_serializer())
@@ -440,20 +489,28 @@ class JsonApiModelViewBuilder:
             if self._after_delete_callback is not None:
                 self._after_delete_callback(instance, view.get_serializer())
 
+        def _retrieve(view, instance):
+            instance = view.get_object()
+            serializer = view.get_serializer(instance)
+            return Response(serializer.data)
+
         @exception_handler
+        @self._get_decorator
         def perform_get(view, instance, *args, **kwargs):
-            response = super(view.__class__, view).retrieve(instance, *args, **kwargs)
+            response = _retrieve(view, instance)
             if self._after_get_callback is not None:
                 response.data = self._after_get_callback(view.request, response.data)
             return response
 
         @exception_handler
+        @self._update_decorator
         def perform_update(view, serializer):
             instance = serializer.save()
             if self._after_update_callback is not None:
                 self._after_update_callback(view.request, instance, serializer)
 
         @exception_handler
+        @self._list_decorator
         def perform_list(view, request, *args, **kwargs):
             queryset = view.filter_queryset(view.get_queryset())
 
@@ -461,15 +518,16 @@ class JsonApiModelViewBuilder:
                 queryset = self._before_list_callback(request, queryset)
 
             page = view.paginate_queryset(queryset)
+
+            serializer = view.get_serializer(page if page is not None else queryset, many=True)
             if page is not None:
-                serializer = view.get_serializer(page, many=True)
                 response = view.get_paginated_response(serializer.data)
             else:
-                serializer = view.get_serializer(queryset, many=True)
                 response = Response(serializer.data)
 
             if self._after_list_callback is not None:
-                response.data = self._after_list_callback(request, response.data)
+                response.data = self._after_list_callback(view.request, response.data)
+
             return response
 
         class Renderer(JSONRenderer):
@@ -477,15 +535,26 @@ class JsonApiModelViewBuilder:
                 response = super(Renderer, inner_self).render(
                     data, accepted_media_type, renderer_context
                 )
+
+                try:
+                    res = ujson.loads(response)
+                    if res.get('data', {}).get('included'):
+                        res['included'] = res['data']['included']
+                        # del res['data']['included']
+                    response = ujson.dumps(res)
+                except Exception as exc:
+                    pass
+
                 if self._before_raw_response:
-                    response = self._before_raw_response(response)
+                    response = self._before_raw_response(response, renderer_context)
                 if not isinstance(response, (bytes, bytearray)):
                     return str.encode(response)
                 return response
 
-        custom_paginator = type(f'PaginatorOf{self._page_size or DEFAULT_PAGE_SIZE}Pages', (LimitedJsonApiPageNumberPagination,), {
-            'page_size': self._page_size or DEFAULT_PAGE_SIZE
-        })
+        custom_paginator = type(f'PaginatorOf{self._page_size or DEFAULT_PAGE_SIZE}Pages',
+                                (LimitedJsonApiPageNumberPagination,), {
+                                    'page_size': self._page_size or DEFAULT_PAGE_SIZE
+                                })
 
         base_model_view_set = type(f'{self._resource_name}JSONApiModelViewSet{self._api_version}', (ModelViewSet,), {
             'renderer_classes': (Renderer,),
@@ -631,6 +700,11 @@ class JsonApiResourceViewBuilder:
                  is_admin: Optional[bool] = False,
                  always_include: Optional[bool] = False,
                  only_callbacks: Optional[bool] = False,
+                 create_decorator: Optional[Callable[[Callable], Callable]] = DEFAULT_VOID_DECORATOR,
+                 update_decorator: Optional[Callable[[Callable], Callable]] = DEFAULT_VOID_DECORATOR,
+                 get_decorator: Optional[Callable[[Callable], Callable]] = DEFAULT_VOID_DECORATOR,
+                 list_decorator: Optional[Callable[[Callable], Callable]] = DEFAULT_VOID_DECORATOR,
+                 delete_decorator: Optional[Callable[[Callable], Callable]] = DEFAULT_VOID_DECORATOR,
                  page_size: int = DEFAULT_PAGE_SIZE):
         self._allowed_methods = [*allowed_methods]
         self._resource_name = action_name
@@ -646,9 +720,15 @@ class JsonApiResourceViewBuilder:
         self._on_list_callback = None
         self._on_get_callback = None
         self._before_raw_response = None
+        self._create_decorator = create_decorator
+        self._update_decorator = update_decorator
+        self._get_decorator = get_decorator
+        self._list_decorator = list_decorator
+        self._delete_decorator = delete_decorator
         self._always_include = always_include
         self._is_admin = is_admin \
-                         or (len(self._permission_classes) > 0 and any(getattr(pc, 'admin', False) for pc in self._permission_classes))
+                         or (len(self._permission_classes) > 0 and any(
+            getattr(pc, 'admin', False) for pc in self._permission_classes))
         self._page_size = page_size
         self._only_callbacks = only_callbacks
         self.settings = JsonApiGlobalSettings()
@@ -669,28 +749,28 @@ class JsonApiResourceViewBuilder:
 
     def on_create(self,
                   create_callback: Callable[[Request], Tuple[Dict, str, int]] = None) -> 'JsonApiResourceViewBuilder':
-        self._on_create_callback = create_callback
+        self._on_create_callback = self._create_decorator(create_callback)
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_POST)
         return self
 
     def on_update(self, update_callback: Callable[[Request], Tuple[Dict, int]] = None) -> 'JsonApiResourceViewBuilder':
-        self._on_update_callback = update_callback
+        self._on_update_callback = self._update_decorator(update_callback)
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_PATCH)
         return self
 
     def on_delete(self, delete_callback: Callable[[Request], Tuple[int]] = None) -> 'JsonApiResourceViewBuilder':
-        self._on_delete_callback = delete_callback
+        self._on_delete_callback = self._delete_decorator(delete_callback)
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_DELETE)
         return self
 
     def on_list(self,
                 list_callback: Callable[[Request], Tuple[List, List, int, int]] = None) -> 'JsonApiResourceViewBuilder':
-        self._on_list_callback = list_callback
+        self._on_list_callback = self._list_decorator(list_callback)
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
         return self
 
     def on_get(self, get_callback: Callable[[Request], Tuple[Dict, List, int]] = None) -> 'JsonApiResourceViewBuilder':
-        self._on_get_callback = get_callback
+        self._on_get_callback = self._get_decorator(get_callback)
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
         return self
 
@@ -806,15 +886,26 @@ class JsonApiResourceViewBuilder:
                 response = super(Renderer, inner_self).render(
                     data, accepted_media_type, renderer_context
                 )
+
+                try:
+                    res = ujson.loads(response)
+                    if res.get('data', {}).get('included'):
+                        res['included'] = res['data']['included']
+                        # del res['data']['included']
+                    response = ujson.dumps(res)
+                except Exception as exc:
+                    pass
+
                 if self._before_raw_response:
-                    response = self._before_raw_response(response)
+                    response = self._before_raw_response(response, renderer_context)
                 if not isinstance(response, (bytes, bytearray)):
                     return str.encode(response)
                 return response
 
-        custom_paginator = type(f'PaginatorOf{self._page_size or DEFAULT_PAGE_SIZE}Pages', (LimitedJsonApiPageNumberPagination,), {
-            'page_size': self._page_size or DEFAULT_PAGE_SIZE
-        })
+        custom_paginator = type(f'PaginatorOf{self._page_size or DEFAULT_PAGE_SIZE}Pages',
+                                (LimitedJsonApiPageNumberPagination,), {
+                                    'page_size': self._page_size or DEFAULT_PAGE_SIZE
+                                })
 
         patch_view_set = None
         if any([self._on_update_callback, self._on_delete_callback, self._on_get_callback]) or not self._only_callbacks:
@@ -906,7 +997,7 @@ def json_api_view(resource_name: str,
                   authentication_classes: Optional[Sequence[Type[BaseAuthentication]]] = None,
                   multiple_resource: Optional[bool] = True,
                   raw_items: Optional[bool] = False,
-                  page_size: Optional[int] = 50,
+                  page_size: Optional[int] = DEFAULT_PAGE_SIZE,
                   is_admin: Optional[bool] = False,
                   always_include: Optional[bool] = False,
                   *args, **kwargs) -> FunctionType:
@@ -937,5 +1028,43 @@ def json_api_view(resource_name: str,
     return decorator
 
 
+def json_api_view_multi_endpoint(resource_name: str,
+                                 api_version: Optional[str] = '',
+                                 allowed_methods: Optional[List[str]] = json_api_spec_http_methods.HTTP_ACTIONS,
+                                 permission_classes: Optional[Sequence[Type[BasePermission]]] = None,
+                                 authentication_classes: Optional[Sequence[Type[BaseAuthentication]]] = None,
+                                 raw_items: Optional[bool] = False,
+                                 page_size: Optional[int] = 50,
+                                 is_admin: Optional[bool] = False,
+                                 always_include: Optional[bool] = False,
+                                 *args, **kwargs) -> FunctionType:
+    def decorator(func: Callable[[], Tuple[Callable, Callable, Callable, Callable, Callable]]):
+        on_list, on_get, on_create, on_delete, on_update = func()
+        builder = JsonApiResourceViewBuilder(action_name=resource_name,
+                                             api_version=api_version,
+                                             allowed_methods=allowed_methods,
+                                             permission_classes=permission_classes,
+                                             authentication_classes=authentication_classes,
+                                             raw_items=raw_items,
+                                             page_size=page_size,
+                                             only_callbacks=True,
+                                             is_admin=is_admin,
+                                             always_include=always_include)
+        if json_api_spec_http_methods.HTTP_GET in allowed_methods and on_list:
+            builder = builder.on_list(list_callback=on_list)
+        if json_api_spec_http_methods.HTTP_GET in allowed_methods and on_get:
+            builder = builder.on_get(get_callback=on_get)
+        if json_api_spec_http_methods.HTTP_POST in allowed_methods and on_create:
+            builder = builder.on_create(create_callback=on_create)
+        if json_api_spec_http_methods.HTTP_DELETE in allowed_methods and on_delete:
+            builder = builder.on_delete(delete_callback=on_delete)
+        if json_api_spec_http_methods.HTTP_PATCH in allowed_methods and on_update:
+            builder = builder.on_update(update_callback=on_update)
+
+        return builder
+
+    return decorator
+
+
 # Backwards Compatibility Support
 JsonApiViewBuilder = JsonApiModelViewBuilder
